import javafx.application.Application;
// import javafx.beans.property.Property;
// import javafx.beans.property.DoubleProperty;
import javafx.event.EventHandler;
import javafx.geometry.Point2D;
import javafx.scene.Group;
import javafx.scene.Scene;
import javafx.scene.input.KeyCode;
import javafx.scene.input.KeyEvent;
import javafx.scene.input.MouseEvent;
import javafx.scene.paint.Color;
import javafx.scene.shape.*;
import javafx.stage.Stage;

import static javafx.beans.binding.Bindings.*;

import java.util.ArrayList;
import java.util.List;

public class MasterSlaveWithBinding extends Application {

    Point2D anchorPt;
    Point currentRedPoint;
    Point currentBluePoint;
    Point lastRedPoint;

    @Override
    public void start(final Stage masterStage) throws Exception {
        final Group root = new Group();

        masterStage.setTitle("Binding Master");
        final Scene scene = new Scene(root, 600, 450);
        masterStage.setScene(scene);

        final double cSceneX = 0.5 * scene.getWidth();
        final double cSceneY = 0.5 * scene.getHeight();

        final Circle masterSphere = CircleBuilder.create()
                .centerX(cSceneX - 150)
                .centerY(cSceneY - 125)
                .radius(20)
                .fill(Color.RED)
                .build();

        System.out.printf("(master ball is at (%f,%f)%n",
                masterSphere.getCenterX(),
                masterSphere.getCenterY());

        final Circle slaveSphere = CircleBuilder.create()
                // .centerX(2 * cSceneX - masterSphere.getCenterX())
                // .centerY(2 * cSceneY - masterSphere.getCenterY())
                .radius(20)
                .fill(Color.BLUE)
                .build();
		
		slaveSphere.centerXProperty()
			.bind(subtract(scene.widthProperty(), masterSphere.centerXProperty()));
		slaveSphere.centerYProperty()
			.bind(subtract(scene.heightProperty(), masterSphere.centerYProperty()));

        // add spheres
        root.getChildren().addAll(masterSphere, slaveSphere);

        // make red sphere draggable, and blue sphere mirror red's path
        masterSphere.setOnMouseDragged(new EventHandler<MouseEvent>() {
            @Override
            public void handle(MouseEvent me) {
                // this is for dragging
                double mex = me.getX();
                double mey = me.getY();
                masterSphere.setCenterX(mex);
                masterSphere.setCenterY(mey);
                // these are not needed when the two spheres' X,Y properties are bound
                // slaveSphere.setCenterX(2 * cSceneX - me.getX());
                // slaveSphere.setCenterY(2 * cSceneY - me.getY());
                // System.out.printf("the sphere is in (%f,%f)%n", mex, mey);
            }
        });

        scene.setOnKeyPressed(new EventHandler<KeyEvent>() {
            @Override
            public void handle(KeyEvent ke) {
                KeyCode code = ke.getCode();
                if (code == KeyCode.UP)
                    masterSphere.setCenterY(masterSphere.getCenterY() - 5);
                else if (code == KeyCode.DOWN)
                    masterSphere.setCenterY(masterSphere.getCenterY() + 5);
                else if (code == KeyCode.RIGHT)
                    masterSphere.setCenterX(masterSphere.getCenterX() + 5);
                else if (code == KeyCode.LEFT)
                    masterSphere.setCenterX(masterSphere.getCenterX() - 5);
            }
        });
		
        masterStage.show();
    }
	
	public static class Point {

	    public double x, y;

	    public Point(double x, double y) {
	        this.x = x;
	        this.y = y;
	    }

	    public static Point makePoint(double x, double y) {
	        return new Point(x, y);
	    }
	}

    public static void main(String[] args) {
        launch(args);
    }
}
